(function (global, factory) {

    "use strict";

    if (typeof module === "object" && typeof module.exports === "object") {

        module.exports = global.document ?
            factory(global, true) :
            function (w) {
                if (!w.document) {
                    throw new Error("zyapp requires a window with a document");
                }
                return factory(w);
            };
    } else {
        factory(global);
    }

    // Pass this if window is not defined yet
})(typeof window !== "undefined" ? window : this, function (window, noGlobal) {
    "use strict";

    //定义app的js交互端口
    var wst = window.wst;
    var ovgap = window.ov_gap = {
        callbackId: Math.floor(Math.random() * 2000000000),
        callbacks: {},
        commandQueue: [],
        groupId: Math.floor(Math.random() * 300),
        groups: {},
        listeners: {},
        invoke: function (cmd, params, onSuccess, onFail) {
            if (!cmd) cmd = "defaultCommand";
            if (!params) params = {};
            this.callbackId++;
            this.callbacks[this.callbackId] = {
                success: onSuccess,
                fail: onFail
            };
            var rurl = "ovgap://" + cmd + "/" + JSON.stringify(params) + "/" + this.callbackId;
            document.location = rurl;
        },
        dispatchCommand: function (cmd, params, onSuccess, onFail) {
            if (!cmd) cmd = "defaultCommand";
            if (!params) params = {};
            this.callbackId++;
            this.callbacks[this.callbackId] = {
                success: onSuccess,
                fail: onFail
            };
            var command = cmd + "/" + JSON.stringify(params) + "/" + this.callbackId;
            this.commandQueue.push(command);
        },
        fetchNativeCommands: function () {
            var json = JSON.stringify(this.commandQueue);
            this.commandQueue = [];
            return json;
        },
        activate: function () {
            document.location = "ovgap://ready";
        },
        // return group ID
        createGroup: function () {
            this.groupId++;
            this.groups[this.groupId] = [];
            return this.groupId;
        },
        dispatchCommandInGroup: function (cmd, params, onSuccess, onFail, groupId) {
            if (!this.groups[groupId]) return false;

            if (!cmd) cmd = "defaultCommand";
            if (!params) params = {};
            this.callbackId++;
            this.callbacks[this.callbackId] = {
                success: onSuccess,
                fail: onFail
            };
            var command = cmd + "/" + JSON.stringify(params) + "/" + this.callbackId;
            this.groups[groupId].push(command);
            return true;
        },
        activateGroup: function (groupId) {
            if (!this.groups[groupId]) return false;
            document.location = "ovgap://group/" + groupId;
        },
        fetchNativeGroupCommands: function (groupId) {
            if (!this.groups[groupId]) return [];
            var json = JSON.stringify(this.groups[groupId]);
            this.groups[groupId] = [];
            return json;
        },
        callbackSuccess: function (callbackId, params) {
            try {
                this.callbackFromNative(callbackId, params, true);
            } catch (e) {
                console.log("Error in error callback: " + callbackId + " = " + e);
            }
        },
        callbackError: function (callbackId, params) {
            try {
                this.callbackFromNative(callbackId, params, false);
            } catch (e) {
                console.log("Error in error callback: " + callbackId + " = " + e);
            }
        },
        callbackFromNative: function (callbackId, params, isSuccess) {
            var callback = this.callbacks[callbackId];
            if (callback) {
                if (isSuccess) {
                    callback.success && callback.success(callbackId, params);
                } else {
                    callback.fail && callback.fail(callbackId, params);
                }
                delete this.callbacks[callbackId];
            };
        },
        addGapListener: function (listenId, onSuccess, onFail) {
            if (!listenId || !onSuccess || !onFail) return;
            this.listeners[listenId] = {
                success: onSuccess,
                fail: onFail
            };
        },
        removeListener: function (listenId) {
            if (!this.listeners[listenId]) return;
            this.listeners[listenId] = null;
        },
        triggerListenerSuccess: function (listenId, params) {
            if (!this.listeners[listenId]) return;
            var listener = this.listeners[listenId];
            listener.success && listener.success(listenId, params);
        },
        triggerListenerFail: function (listenId, params) {
            if (!this.listeners[listenId]) return;
            var listener = this.listeners[listenId];
            listener.fail && listener.fail(listenId, params);
        }
    };

    var version = '1.0';
    var queue = [],
        running = false,
        browser = {
            version: function () {
                var u = navigator.userAgent;
                return {         //移动终端浏览器版本信息
                    //trident: u.indexOf('Trident') > -1, //IE内核
                    //presto: u.indexOf('Presto') > -1, //opera内核
                    //webKit: u.indexOf('AppleWebkit') > -1, //苹果、谷歌内核
                    //gecko: u.indexOf('Gecko') > -1 && u.indexOf('Khtml') == -1, //火狐内核
                    //mobile: !!u.match(/Applewebkit.*Mobile.*/), //是否为移动终端
                    ios: !!u.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/), //ios终端
                    android: !!u.match(/Android|Linux/), //android终端或uc浏览器
                    //iPhone: u.indexOf('iphone') > -1, //是否为iPhone或者QQHD浏览器
                    //iPad: u.indexOf('ipad') > -1, //是否iPad
                    zyapp: !!u.match(/\bzyapp\b/),
                    sgs: parseFloat(u.match(/\bSGS\b\/.*?(?=( |$))/) ? u.replace(/.*\bSGS\/(?=.*(?=( |$)))/, '').replace(/ .*/, '') : '0'),
                    ats: parseFloat(u.match(/\bATS\b\/.*?(?=( |$))/) ? u.replace(/.*\bATS\/(?=.*(?=( |$)))/, '').replace(/ .*/, '') : '0')
                };
            }()
        };
    var zyapp = function () {
        return new zyapp.fn.init();
    };
    zyapp.fn = zyapp.prototype = {};
    var init = zyapp.fn.init = function () {
        return this;
    };
    init.prototype = zyapp.fn;
    zyapp.prototype.browser = browser;

    function onWebComponents(module) {

        console.log('onWebComponents$' + JSON.stringify(module || {}));
        //if (!browser.version.mqkt && !browser.version.sgs && !browser.version.ats)
        //return;
        if (browser.version.android) {
            wst.startAppActivity('onWebComponents$' + JSON.stringify(module || {}));
        } else if (browser.version.ios) {
            ovgap.dispatchCommand('onWebComponents', { result: JSON.parse(JSON.stringify((module || {}))) }, null, null);
            ovgap.activate();
        }
    };
    function activate() {
        running = true;
        var m = queue.splice(0, 1);
        if (m.length > 0) {
            onWebComponents(m[0]);
            setTimeout(activate, zyapp().setting.interval);
        } else {
            running = false;
        }
    };

    zyapp.prototype.handle = function (id, title, data, api) {
        var module = {
            id: id,
            title: title,
            data: !!data ? (!!data.code ? data : { code: 0, info: '', result: data }) : {},
            api: api || {}
        };
        queue.splice(queue.length, 0, module);
        if (!running) {components
            activate();
        }
        return this;
    };

    zyapp.prototype.setting = {
        interval: 100
    };


    var components = {
        logout: { id: 201, title: '退出登录' },
        newWindow: { id: 202, title: '新建窗口' },
        closeWindow: { id: 203, title: '关闭窗口' },
        share: { id: 204, title: '分享' },
        alert: { id: 205, title: '警告' },
        toast: { id: 206, title: '提示' },
        uploadSingleImg: { id: 207, title: '单图上传' },
        uploadImgs: { id: 208, title: '多图上传' },
        showImgs: { id: 209, title: '图片浏览' },
        backKey: { id: 210, title: '返回键' },
        ajax: { id: 211, title: 'api调用' },
        camera:{id:212,title:'拍照'},
        record:{id:213,title:'录音'},
        playVideo:{id:214,title:'播放视频'},
        clientData: {id: 215, title: '获取客户端数据'},
        video: {id: 216, title: '录制视频'},
        delImgs: { id: 217, title: '浏览图片(删除)' },
        rightBtn: { id: 218, title: '右上角' },
        closeAllWindow: { id: 219, title: '关闭全部窗口' },
        datetimePicker: { id: 220, title: '时间选择器' },
        setTitle: { id: 221, title: '设置标题' },
        setNav: { id: 300, title: '设置导航' }
    }

    zyapp.prototype.com = {
        ajax: function (url, data, done, fail, always) {
            
            //url = url.replace("http://h5.czpg.pc118.zgyey.com", "http://w.czpg.pc118.zgyey.com");
            //url = url.replace("http://h5.czpg.pc118.zgyey.com", "http://192.168.0.138:8051");
            //$.ajax({
            //    url: url,
            //    data: data,
            //    dataType: 'json',
            //    success: function (data) {
            //        $(window).trigger(done, data);
            //    },
            //    error: function (xhr, type) {
            //        $(window).trigger(fail);
            //    }
            //})
            //return;

            var module = components.ajax;
            var api = { url: url,data:data, done: done, fail: fail, always: always }
            return zyapp().handle(module.id, module.title, {}, api);
        },
        logout: function (url) {
            var module = components.logout;
            var data = {};
            var api = { url: url };
            return zyapp().handle(module.id, module.title, data, api);
        },
        newWindow: function (url, left, right) {
            if (!!!right && !!left) {
                var t = right
                right = left
                left = t
            }
            var module = components.newWindow
            // if (location.host.indexOf('http') == 0)
            if (!url.match(/[a-zA-z]+:\/\/[^\s]*/)) {
                url = url.indexOf('/') == 0 ? (location.protocol + '//' + location.host + '/' + url) : (location.href.substring(0, location.href.lastIndexOf('/') + 1) + url)
            }
            var data = { url: url, bar: left === null ? undefined : { left: left, right: right || { type: 0 } } }
            var api = {}
            return zyapp().handle(module.id, module.title, data, api)
        },
        closeWindow: function () {
            var module = components.closeWindow;
            var data = {};
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        share: function (url, title, content, img) {
            var module = components.share;
            var data = { url: url, title: title, content: content, img: img };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        alert: function (message) {
            var module = components.alert;
            var data = { message: message };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        toast: function (message) {
            var module = components.toast;
            var data = { message: message };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        uploadSingleImg: function (path, callback) {
            var module = components.uploadSingleImg;
            var data = { path: path, callback: callback };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        uploadImgs: function (count, path, callback) {
            if (!!!callback) {
                callback = path;
                path = '';
            }
            var module = components.uploadImgs;
            var data = { path: path, count: count, callback: callback };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        showImgs: function (img,imgs,index) {
            var module = components.showImgs;
            var data = { img: img, imgs: imgs, index: index };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        backKey: function (callback) {
            var module = components.backKey;
            var data = { callback: callback };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        carema:function (callback) {
            var module = components.camera;
            var data = {callback:callback};
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        record:function (callback) {
            var module = components.record;
            var data = {callback:callback};
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        playVideo:function (url) {
            var module = components.playVideo;
            var data = {url:url};
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        clientData: function (callback) {
            var module = components.clientData;
            var data = {callback: callback};
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        video: function (callback) {
            var module = components.video;
            var data = {callback: callback};
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        delImgs: function (img, imgs, index,callback) {
            var module = components.delImgs;
            var data = { img: img, imgs: imgs, index: index, callback: callback };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        rightBtn: function (type, content, callback) {
            var module = components.rightBtn;
            var data = { type: type, content: content, callback: callback };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        delImgsLeave: function (img, imgs, index, callback) {
            var module = components.delImgsLeave;
            var data = { img: img, imgs: imgs, index: index, callback: callback };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        closeAllWindow: function () {
            var component = components.closeAllWindow;
            var data = {};
            var api = {};
            return zyapp().handle(component.id, component.title, data, api);
        },
        datetimePicker: function (type, value, min, max, callback) {
            var component = components.datetimePicker;
            var data = { type: type, value: value, min: min, max: max, callback: callback };
            var api = {};
            return zyapp().handle(component.id, component.title, data, api);
        },
        setNav: function (left,title,right) {
            var module = components.setNav;
            var data = { left: left, title: title, right: right };
            var api = {};
            return zyapp().handle(module.id, module.title, data, api);
        },
        setTitle: function (title) {
            document.title = title;
            var component = components.setTitle;
            var data = { title: title };
            var api = {};
            return zyapp().handle(component.id, component.title, data, api);
           
        }
    }

    //AMD
    if (typeof define === "function" && define.amd) {
        define("zyapp", [], function () {
            return zyapp;
        });
    }

    if (!noGlobal) {
        window.zyapp = zyapp;
    }

    return zyapp;

});


window.newWindow = function (url) {
    zyapp().com.newWindow(url);
}
window.closeWindow = function () {
    zyapp().com.closeWindow();
}
window.logout = function (url) {
    zyapp().com.logout(url);
}
window.para=function () {
    var s = location.search.replace(/\?/,'').split('&');
    var p = {};
    for(var i=0;i<s.length;i++){
        var t = s[i].match(/(.*[^=])=(.*)/);
        if(!!t)
            p[t[1]] = t[2];
    }
    return p;
}()